@@ -39,34 +39,31 @@ class _Action:
     find, create = 0,1
 
 class AGraph(object):
-    """
-    Class for Graphviz agraph type.
-
-    Example use::
-         
-      >>> G=AGraph()         
-      >>> G=AGraph(directed=True)
-
-      G=AGraph("file.dot")   
-
+    """Class for Graphviz agraph type.
+    
+    Example use
+    
+    >>> G=AGraph()         
+    >>> G=AGraph(directed=True)
+    >>> G=AGraph("file.dot")   
+    
     Graphviz graph keyword parameters are processed so you may add
     them like
-
+    
     >>> G=AGraph(landscape='true',ranksep='0.1')
-
+    
     or alternatively
-
+    
     >>> G=AGraph()
     >>> G.graph_attr.update(landscape='true',ranksep='0.1')
-
+    
     and
-
+    
     >>> G.node_attr.update(color='red')
     >>> G.edge_attr.update(len='2.0',color='blue')
-
+    
     See http://www.graphviz.org/doc/info/attrs.html
     for a list of attributes.
-    
 
     Keyword parameters:
 
@@ -81,18 +78,20 @@ class AGraph(object):
     >>> h=AGraph.handle
     >>> C=AGraph(h)
 
-    name is a string name for the graph        
+    Parameters:: 
 
-    strict=True|False (True for simple graphs)
+      name:    String name for the graph        
 
-    directed=True|False
+      strict: True|False (True for simple graphs)
 
-    data is a dictionary of dictionaries or dictionary of lists
-    representing nodes or edges to load into intial graph
+      directed: True|False
 
-    string is a string containing a dot format graph
+      data: Dictionary of dictionaries or dictionary of lists
+      representing nodes or edges to load into intial graph
 
-    handle is a Swig pointer to an agraph_t data structure
+      string:  String containing a dot format graph
+
+      handle:  Swig pointer to an agraph_t data structure
 
     """
     def __init__(self, thing=None, file=None, name=None,
@@ -112,6 +111,8 @@ class AGraph(object):
             else:
                 file=thing  # assume this is a file name
 
+        self.charset='UTF-8' # default charset encoding 
+
         # input type guessed or specified - now init graph                
         if handle is None:  # the graph pointer (handle)
             self.handle=gv.agraphnew(name,strict,directed) # new graph
@@ -182,7 +183,7 @@ class AGraph(object):
 #        self.add_edge(u,v)
 
     def add_node(self, n, **attr):
-        """Add a single node n to the graph.
+        """Add a single node n.
 
         If n is not a string, conversion to a string will be attempted.
         String conversion will work if n has valid string representation
@@ -207,6 +208,7 @@ class AGraph(object):
 
         """
         if not self._is_string_like(n):  n=str(n)
+        n=n.encode(self.charset)
         try:
             nh=gv.agnode(self.handle,n,_Action.find)
         except KeyError:
@@ -215,7 +217,7 @@ class AGraph(object):
             nh.attr.update(**attr)
 
     def add_nodes_from(self, nbunch, **attr):
-        """Add nodes to graph from a container nbunch.
+        """Add nodes from a container nbunch.
 
         nbunch can be any iterable container such as a list or dictionary
 
@@ -235,7 +237,7 @@ class AGraph(object):
             self.add_node(n,**attr)
 
     def delete_node(self,n):
-        """Delete the single node n from graph.
+        """Delete the single node n.
 
         Attempting to delete a node that isn't in the graph will produce
         an error.
@@ -253,7 +255,7 @@ class AGraph(object):
             raise KeyError("node %s not in graph"%n)
 
     def delete_nodes_from(self,nbunch):
-        """Delete nodes in graph from a container nbunch.
+        """Delete nodes from a container nbunch.
 
         nbunch can be any iterable container such as a list or dictionary
 
@@ -616,7 +618,7 @@ class AGraph(object):
                 while eh is not None:
                     (s,t)=(gv.agtail(eh),gv.aghead(eh))
                     (u,v)=(gv.agnameof(s),gv.agnameof(t))
-                    yield Edge(self,u,v,key=gv.agnameof(key),eh=eh)
+                    yield Edge(self,u,v,key=gv.agnameof(eh),eh=eh)
                     eh=gv.agnxtin(eh)
         raise StopIteration
 
@@ -812,7 +814,7 @@ class AGraph(object):
                 
     def number_of_edges(self):
         """Return the number of edges in the graph."""
-        return sum(self.degree())/2
+        return gv.agnedges(self.handle)
 
     def size(self):
         """Return the number of edges in the graph."""
@@ -824,9 +826,17 @@ class AGraph(object):
         self.edge_attr.clear()
         self.node_attr.clear()
         self.graph_attr.clear()
-        self.delete_edges_from(self.edges())
-        self.delete_nodes_from(self.nodes())
+        # now "close" existing graph and create a new graph
+        name=gv.agnameof(self.handle)
+        strict=self.is_strict()
+        directed=self.is_directed()
+        gv.agclose(self.handle) 
+        self.handle=gv.agraphnew(name,strict,directed) 
 
+    def close(self):
+        # may be useful to clean up graphviz data
+        # this should completely remove all of the existing graphviz data
+        gv.agclose(self.handle)
 
     def copy(self):
         """Return a copy of the graph."""
@@ -871,11 +881,11 @@ class AGraph(object):
         return bunch
 
 
-    def add_subgraph(self, nbunch=None, name=None):
+    def add_subgraph(self, nbunch=None, name=None, **attr):
         """Return subgraph induced by nodes in nbunch.
         """
         handle=gv.agsubg(self.handle,name,_Action.create)
-        H=self.__class__(strict=self.is_strict(),directed=self.is_directed(),handle=handle,name=name)
+        H=self.__class__(strict=self.is_strict(),directed=self.is_directed(),handle=handle,name=name,**attr)
         if nbunch is None: return H
         # add induced subgraph on nodes in nbunch
         bunch=self.prepare_nbunch(nbunch)
@@ -1070,20 +1080,17 @@ class AGraph(object):
         return self.to_string()
 
     def from_string(self,string):
-        """ 
-        Load a graph from a string in dot format. Overwrites any existing graph.
+        """Load a graph from a string in dot format.
 
+        Overwrites any existing graph.
+        
         To make a new graph from a string use
         
-        >>> s='digraph {\n1 -> 2\n;}\n'
+        >>> s='digraph {1 -> 2}'
         >>> A=AGraph()
         >>> A.from_string(s)
-
-        or
-
         >>> A=AGraph(string=s) # specify s is a string
         >>> A=AGraph(s)  # s assumed to be a string during initialization
-
         """
         from tempfile import TemporaryFile
         fh = TemporaryFile()
@@ -1145,9 +1152,9 @@ class AGraph(object):
 
         if data==[]:
             raise IOError("".join(errors))
-# errors should be reported here somehow - is this the right way?
-#        if len(errors)>0:
-#            warnings.warn("".join(errors),UserWarning) 
+
+        if len(errors)>0:
+            raise IOError("".join(errors))
         return "".join(data)
 
 
@@ -1222,12 +1229,15 @@ class AGraph(object):
         on the file extension of path.  If that fails the format keyword
         will be used.  
         
-        Formats::
+        Formats (not all may be available on every system depending on
+        how Graphviz was built)
+
+            'canon', 'cmap', 'cmapx', 'cmapx_np', 'dia', 'dot',
+            'fig', 'gd', 'gd2', 'gif', 'hpgl', 'imap', 'imap_np',
+            'ismap', 'jpe', 'jpeg', 'jpg', 'mif', 'mp', 'pcl', 'pdf',
+            'pic', 'plain', 'plain-ext', 'png', 'ps', 'ps2', 'svg',
+            'svgz', 'vml', 'vmlz', 'vrml', 'vtx', 'wbmp', 'xdot', 'xlib'
 
-             ['canon','dot','xdot','cmap','dia','fig','gd','gd2',
-             'gif','hpgl','imap','cmapx','ismap','jpg','jpeg',
-             'mif','mp','pcl','pic','plain','plain-ext','png','ps',
-             'ps2','svg','svgz','vrml','vtx','wbmp']
 
         If prog is not specified and the graph has positions
         (see layout()) then no additional graph positioning will
@@ -1260,16 +1270,17 @@ class AGraph(object):
 
         if format is None or format=='':
             format = 'dot'
-        else:
-            formats=dict.fromkeys(
-                ['canon','dot','xdot','cmap','dia','fig','gd','gd2',\
-                 'gif','hpgl','imap','cmapx','ismap','jpg','jpeg',\
-                 'mif','mp','pcl','pic','plain','plain-ext','png','ps',\
-                 'ps2','svg','svgz','vrml','vtx','wbmp'])
-
-            if not format in formats:
-                raise ValueError("format %s is not one of %s"%\
-                                 (format,', '.join(formats.keys()))) 
+#         else:
+#             formats=dict.fromkeys(
+#                 ['canon', 'cmap', 'cmapx', 'cmapx_np', 'dia', 'dot',\
+#                      'fig', 'gd', 'gd2', 'gif', 'hpgl', 'imap', 'imap_np',\
+#                      'ismap', 'jpe', 'jpeg', 'jpg', 'mif', 'mp', 'pcl', 'pdf',\
+#                      'pic', 'plain', 'plain-ext', 'png', 'ps', 'ps2', 'svg',\
+#                      'svgz', 'vml', 'vmlz', 'vrml', 'vtx', 'wbmp', 'xdot', \
+#                      'xlib'])
+#             if not format in formats:
+#                 raise ValueError("format %s is not one of %s"%\
+#                                  (format,', '.join(formats.keys()))) 
 
         if prog is None:
             try:
@@ -1332,7 +1343,7 @@ class AGraph(object):
                  fh=os.popen("bzcat "+path) # probably not portable
             else:
                 fh = file(path,mode=mode)
-        elif hasattr(path, 'seek'):
+        elif hasattr(path, 'write'):
             fh = path
         else:
             raise TypeError('path must be a string or file handle')
@@ -1496,11 +1507,25 @@ class Attribute(UserDict.DictMixin):
         self.__dict__['type']=atype
 
     def __setitem__(self, name, value):
-        try:
-            gv.agattr(self.handle,self.type,name,value)
+        try: # try to set the attribute
+            gv.agset(self.handle,name,value)
+        except KeyError: # not in root graph default attribute set
+            ghandle=gv.agroot(self.handle) # get root graph
+            # check if this is the root graph and set default for all items
+            if self.handle==ghandle:
+                gv.agattr(ghandle,self.type,name,value) 
+            else: # this is not the root graph
+                try:
+                    # set graph default to be empty string
+                    gv.agattr(ghandle,self.type,name,'') 
+                    # set the attribute for this item
+                    gv.agset(self.handle,name,value)
+                except KeyError: # not sucessful
+                    pass
         except TypeError:
             raise "Attribute value must be a string"
 
+
     def __getitem__(self, name):
         try:
             ah=gv.agattr(self.handle,self.type,name,None)
