@@ -9,8 +9,32 @@
 #    All rights reserved, see LICENSE for details.
 
 
+import os
+import re
+
+
+def get_svn_revision():
+    rev = None
+    path ="."
+    entries_path = '%s/.svn/entries' % path
+    if os.path.exists(entries_path):
+        entries = open(entries_path, 'r').read()
+        # Versions >= 7 of the entries file are flat text.  The first line is
+        # the version number. The next set of digits after 'dir' is the revision.
+        if re.match('(\d+)', entries):
+            rev_match = re.search('\d+\s+dir\s+(\d+)', entries)
+            if rev_match:
+                rev = rev_match.groups()[0]
+    if rev:
+        return 'dev%s' % rev
+    return None
+
 name = 'pygraphviz'
-version = '0.36'
+version = '0.99'
+revision = None
+if revision is not None:
+    version+=".%s"%revision
+
 description = "Python interface to Graphviz"
 long_description = """\
 A Python wrapper for the Graphviz Agraph data structure.
